from __future__ import annotations
import os
from typing import Optional
import ckan.plugins.toolkit as tk
from ckan.logic import validate
from ckan.lib.uploader import get_uploader, get_storage_path

from ckanext.toolbelt.decorators import Collector

from ckanext.files.model import File
from . import schema

action, get_actions = Collector("files").split()

CONFIG_SIZE = "ckanext.files.kind.{kind}.max_size"
DEFAULT_SIZE = 2


def files_uploader(kind: str, old_filename: Optional[str] = None):
    return get_uploader(kind, old_filename)


@action
@validate(schema.file_create)
def file_create(context, data_dict):
    tk.check_access("files_file_create", context, data_dict)

    uploader = files_uploader(data_dict["kind"])
    uploader.update_data_dict(data_dict, "path", "upload", None)

    max_size = tk.asint(tk.config.get(CONFIG_SIZE.format(kind=data_dict["kind"]), DEFAULT_SIZE))
    uploader.upload(max_size)

    # TODO: try not to rely on hardcoded segments
    data_dict["path"] = os.path.relpath(uploader.filepath, os.path.join(get_storage_path(), "storage"))

    file = File(**data_dict)
    context["session"].add(file)
    context["session"].commit()
    return file.dictize(context)


@action
@validate(schema.file_update)
def file_update(context, data_dict):
    tk.check_access("files_file_delete", context, data_dict)
    file = (
        context["session"]
        .query(File)
        .filter_by(id=data_dict["id"])
        .one_or_none()
    )
    if not file:
        raise tk.ObjectNotFound("File not found")

    for attr, value in data_dict.items():
        setattr(file, attr, value)
    context["session"].commit()
    return file.dictize(context)


@action
def file_delete(context, data_dict):
    id_ = tk.get_or_bust(data_dict, "id")
    tk.check_access("files_file_delete", context, data_dict)
    file = context["session"].query(File).filter_by(id=id_).one_or_none()
    if not file:
        raise tk.ObjectNotFound("File not found")

    context["session"].delete(file)
    context["session"].commit()
    return True


@action
def file_show(context, data_dict):
    id_ = tk.get_or_bust(data_dict, "id")
    tk.check_access("files_file_show", context, data_dict)

    query = context["session"].query(File)
    file = query.filter_by(id=id_).one_or_none()
    if not file:
        file = query.filter_by(name=id_).one_or_none()
    if not file:
        raise tk.ObjectNotFound("File not found")

    return file.dictize(context)



from ckanext.toolbelt.decorators import Collector

auth, get_auth_functions = Collector("files").split()


@auth
def file_create(context, data_dict):
    {"success": False}


@auth
def file_update(context, data_dict):
    {"success": False}


@auth
def file_delete(context, data_dict):
    {"success": False}


@auth
def file_show(context, data_dict):
    {"success": False}



import ckan.plugins as plugins


from .logic import action, auth


class FilesPlugin(plugins.SingletonPlugin):
    plugins.implements(plugins.IActions)
    plugins.implements(plugins.IAuthFunctions)

    # IActions
    def get_actions(self):
        return action.get_actions()

    # IAuthFunctions
    def get_auth_functions(self):
        return auth.get_auth_functions()



from __future__ import annotations

import datetime


from sqlalchemy import Column, UnicodeText, DateTime
from sqlalchemy.dialects.postgresql import JSONB

import ckan.plugins.toolkit as tk
from ckan.model.types import make_uuid
from ckan.lib.dictization import table_dictize
from .base import Base


class File(Base):
    __tablename__ = "files_file"
    id = Column(UnicodeText, primary_key=True, default=make_uuid)
    name = Column(UnicodeText, nullable=False)
    path = Column(UnicodeText, nullable=False)
    kind = Column(UnicodeText, nullable=False)
    uploaded_at = Column(
        DateTime, nullable=False, default=datetime.datetime.utcnow
    )
    extras = Column(JSONB)

    def dictize(self, context):
        result = table_dictize(self, context)
        result["url"] = tk.h.url_for_static(result["path"], qualified=True)
        return result



import ckan.plugins.toolkit as tk
from ckan.logic.schema import validator_args


CONFIG_KIND = "ckanext.files.default.kind"
DEFAULT_KIND = "ckanext_files_file"


@validator_args
def file_create(
        ignore_empty, dict_only, not_empty, unicode_safe, default, ignore, not_missing
):
    default_kind = tk.config.get(CONFIG_KIND, DEFAULT_KIND)
    return {
        "name": [not_empty, unicode_safe],
        "upload": [not_missing],
        "kind": [default(default_kind), unicode_safe],
        "extras": [ignore_empty, dict_only],
        "__extras": [ignore],
    }


@validator_args
def file_update(not_empty, ignore_missing, unicode_safe, dict_only, ignore):
    return {
        "id": [not_empty],
        "name": [ignore_missing, unicode_safe],
        "upload": [ignore_missing],
        "kind": [ignore_missing, unicode_safe],
        "extras": [ignore_missing, dict_only],
        "__extras": [ignore],
    }
